﻿using System;
using HarmonyLib;
using Amplitude;
using Amplitude.Mercury.Simulation;
using Amplitude.Mercury.Interop;

namespace AOM.Humankind.EraStarCatchupFame
{
    [HarmonyPatch(typeof(DepartmentOfDevelopment))]
    public class DepartmentOfDevelopment_Patch
    {

        [HarmonyPatch(nameof(GainFame))]
        [HarmonyPostfix]
        public static void GainFame(DepartmentOfDevelopment __instance, FixedPoint gain, bool raiseSimulationEvents)
        {
            var CurrentEraIndex = __instance.CurrentEraIndex();
            var majorEmpire = __instance.majorEmpire();
            var turn = R.SandboxManager_Sandbox().Turn();
            var player = GetPlayerName(majorEmpire);
            var total = majorEmpire.FameScore().Value;

            var stackTrace = new System.Diagnostics.StackTrace();
            var caller = stackTrace.GetFrame(2).GetMethod().Name;
            EraStarCatchupFame.Instance.LogFame(turn, player, caller, gain, total);
        }


        [HarmonyPatch(nameof(NewTurnBegin_RefreshEraStarCatchup))]
        [HarmonyPrefix]
        public static bool NewTurnBegin_RefreshEraStarCatchup(DepartmentOfDevelopment __instance, SimulationPasses.PassContext context, string name)
        {
            ref var EraStarCatchupInfo = ref AccessTools.FieldRefAccess<DepartmentOfDevelopment, EraStarCatchupInfo>(__instance, typeof(EraStarCatchupInfo).Name);
            var CurrentEraIndex = __instance.CurrentEraIndex();
            var majorEmpire = __instance.majorEmpire();

            if (StaticString.IsNullOrEmpty(EraStarCatchupInfo.EraStarCatchupDefinitionName) || CurrentEraIndex == 0)
            {
                return false;
            }
            EraStarCatchupInfo.PointsPerTurn = __instance.ComputeEraStarCatchupPointsPerTurn();
            EraStarCatchupInfo.Score += EraStarCatchupInfo.PointsPerTurn;
            if (EraStarCatchupInfo.Score > EraStarCatchupInfo.Threshold)
            {
                EraStarCatchupInfo.Score -= EraStarCatchupInfo.Threshold;
                if (EraStarCatchupInfo.UnlockTurn == null || EraStarCatchupInfo.UnlockTurn.Length < EraStarCatchupInfo.Level + 1)
                {
                    Array.Resize(ref EraStarCatchupInfo.UnlockTurn, EraStarCatchupInfo.Level + 1);
                }
                EraStarCatchupInfo.UnlockTurn[EraStarCatchupInfo.Level] = R.SandboxManager_Sandbox().Turn();
                EraStarCatchupInfo.Level++;
                majorEmpire.EraStarsCount.Value += 1;
                AddKetchupFame(__instance);
                __instance.SetupEvolutionPeriodicNotificationIfRequired();
                EraStarCatchupInfo.PointsPerTurn = __instance.ComputeEraStarCatchupPointsPerTurn();
            }
            EraStarCatchupInfo.Frame = R.SandboxManager_Sandbox().Turn();
            return false;
        }

        internal static string GetPlayerName(MajorEmpire majorEmpire)
        {
            return majorEmpire.IsControlledByHuman() ? majorEmpire.PlayerIdentifier().UserName :
            majorEmpire.PersonaName();
        }
        private static void AddKetchupFame(DepartmentOfDevelopment instance)
        {
            var fame = EraStarCatchupFame.Instance.FameBonus.Value;
            instance.GainFame((FixedPoint)fame, true);


            EraStarCatchupFame.Instance.LogKetchup(R.SandboxManager_Sandbox().Turn(), GetPlayerName(instance.majorEmpire()));
        }
    }
}
